﻿using log4net;
using Mono.Addins;
using Nini.Config;
using OpenMetaverse;
using OpenSim.Framework;
using OpenSim.Region.Framework.Interfaces;
using OpenSim.Region.Framework.Scenes;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using System.Text;

[assembly: Addin("TextureFetcher", "0.1")]
[assembly: AddinDependency("OpenSim.Region.Framework", OpenSim.VersionInfo.VersionNumber)]
namespace OpenSim.Modules.TextureFetcher
{
    [Extension(Path = "/OpenSim/RegionModules", NodeName = "RegionModule", Id = "RegionFullPerm")]
    class TextureFetcher : INonSharedRegionModule
    {
        private static readonly ILog m_log = LogManager.GetLogger(MethodBase.GetCurrentMethod().DeclaringType);
        Scene m_scene = null;

        public string Name
        {
            get { return "TextureFetcher"; }
        }

        public void AddRegion(Scene scene)
        {
        }

        public void RemoveRegion(Scene scene)
        {
        }

        public void Close()
        {
        }

        public void Initialise(IConfigSource source)
        {

        }

        public Type ReplaceableInterface
        {
            get { return null; }
        }

        public void RegionLoaded(Scene scene)
        {
            m_scene = scene;

            scene.EventManager.OnObjectAddedToScene += AddObject;
            scene.EventManager.OnSceneObjectLoaded += AddObject;
            scene.EventManager.OnIncomingSceneObject += AddObject;
            scene.EventManager.OnSceneObjectPartUpdated += UpdateObject;
            scene.EventManager.OnSceneObjectPartCopy += CopyObject;

            m_log.Info("[" + Name + "] Region '" + scene.Name + "' loaded.");
        }

        private void CopyObject(SceneObjectPart copy, SceneObjectPart original, bool userExposed)
        {
            copyTexturesToInventory(copy);
        }

        private void UpdateObject(SceneObjectPart sop, bool full)
        {
            copyTexturesToInventory(sop);
        }

        private void AddObject(SceneObjectGroup obj)
        {
            foreach(SceneObjectPart _part in obj.Parts)
                copyTexturesToInventory(_part);
        }

        private bool inventoryContainsScripts(SceneObjectPart part)
        {
            foreach (TaskInventoryItem item in part.Inventory.GetInventoryItems())
                if (item.InvType == 10)
                    return true;

            return false;
        }

        private bool isAssetInInventory(SceneObjectPart part, UUID assetID)
        {
            List<TaskInventoryItem> inventarContend = part.Inventory.GetInventoryItems();

            foreach (TaskInventoryItem item in inventarContend)
                if (item.AssetID == assetID)
                    return true;

            return false;
        }

        private void copyTexturesToInventory(SceneObjectPart part)
        {
            if (part.ParentGroup.IsAttachment)
                return;

            if (inventoryContainsScripts(part))
                return;

            m_log.Info("[" + Name + "] Process part '" + part.Name + "'");

            try
            {
                List<UUID> allTextures = new List<UUID>();
                List<TaskInventoryItem> inventoryItems = new List<TaskInventoryItem>();

                //Get all Textures from the scene object
                Primitive.TextureEntry textures = part.Shape.Textures;
                int allSides = part.GetNumberOfSides();
                for (uint i = 0; i < allSides; i++)
                {
                    if (textures.FaceTextures[i] != null)
                    {
                        if (textures.FaceTextures[i].TextureID != null)
                            allTextures.Add(textures.FaceTextures[i].TextureID);

                        if (textures.FaceTextures[i].MaterialID != null)
                            allTextures.Add(textures.FaceTextures[i].MaterialID);
                    }
                }


                //Convert texture uuid list to inventar items.
                foreach (UUID texture in allTextures)
                {
                    if (texture != UUID.Zero)
                    {
                        TaskInventoryItem item = new TaskInventoryItem();
                        item.AssetID = texture;
                        item.ItemID = UUID.Random();
                        item.OwnerID = part.OwnerID;
                        item.CurrentPermissions = 581639;
                        item.Name = "tf-" + texture.ToString();
                        item.Description = "This item was generated by the texture fetcher module.";
                        item.OwnerID = part.OwnerID;
                        item.CreatorID = part.OwnerID;
                        item.LastOwnerID = part.OwnerID;
                        inventoryItems.Add(item);
                    }
                }

                //Check if the item is allready in the inventory and then add it.
                bool updateNeeded = false;
                foreach (TaskInventoryItem item in inventoryItems)
                {
                    if (!isAssetInInventory(part, item.AssetID))
                    {
                        updateNeeded = true;
                        part.Inventory.AddInventoryItem(item, false);
                        m_log.Info("[" + Name + "] Add item '" + item.Name + "' to '" + part.Name + "'.");
                    }
                }

                if (updateNeeded == true)
                {
                    part.SendFullUpdateToAllClients();
                }
            }
            catch(Exception _error)
            {
                m_log.Error("[" + Name + "] ERROR: " + _error.Message);
            }                
        }
    }
}
